
#define _CRT_SECURE_NO_WARNINGS
#include <algorithm>
#include <cstdio>
#include <vector>
#include <set>
#include <iostream>
#include <cassert>
//#include <>
using namespace std;

#define pb push_back
#define fi first
#define se second
const int inf = 1e9;

struct edge {
	int from, to, f, cap, b;
};

const int N = 3e+5 + 20;
vector<edge> net[N];

void add_edge(int from, int to) {
	edge a = { from, to, 0, 1, net[to].size() };
	edge b = { to, from, 0, 0, net[from].size() };
	net[from].pb(a);
	net[to].pb(b);
}

bool used[N];

bool falkerson(int from, int to) {
	if (from == to) {
		return 1;
	}
	if (used[from]) return 0;
	used[from] = 1;
	for (auto &x : net[from]) {
		if (x.f < x.cap && falkerson(x.to, to)) {
			x.f++;
			net[to][x.b].f--;
			return 1;
		}
	}
	return 0;
}

bool go(int from, int to, vector<int> &path) {
	if (from == to) {
		path.pb(to);
		return 1;
	}
	for (auto &x : net[from]) {
		if (x.f == 1 && x.cap == 1 && go(x.to, to, path)) {
			x.f--;
			net[to][x.b].f++;
			path.pb(from);
			return 1;
		}
	}
	return 0;
}

void print_answer(int a, int b, vector<vector<int> > const & G) {
	//cerr << a << ' ' << b << '\n';

	for (int i = 0; i < 3 * G.size(); ++i) {
		net[i].clear();
	}
	for (int i = 0; i < G.size(); ++i) {
		add_edge(2 * i, 2 * i + 1);
		for (auto to : G[i]) {
			add_edge(2 * i + 1, 2 * to);
		}
	}
	for (int i = 0; i < 3; ++i) {
		fill(used, used + G.size() * 3, 0);
		assert(falkerson(2 * a + 1, 2 * b));
	}
	printf("%d %d\n", a + 1, b + 1);
	for (int i = 0; i < 3; ++i) {
		static vector<int> path;
		path.clear();
		go(2 * a + 1, 2 * b, path);
		reverse(path.begin(), path.end());
		printf("%d ", path.size());
		auto last = -10;
		for (auto x : path) {
			if (x / 2 != last) {
				printf("%d ", x / 2 + 1);
			}
			last = x / 2;
		}
		printf("\n");
	}

	// !!!!
}

void dfs_1(int v,
	int pred,
	vector<int> & H,
	vector<int> & p,
	int h,
	vector<vector<int> > const & G) {
	if (H[v] >= 0) {
		return;
	}
	H[v] = h++;
	p[v] = pred;
	for (auto u : G[v]) {
		dfs_1(u, v, H, p, h, G);
	}
}

int lca(int a, int b, vector<int> const & p) {
	set<int> G;
	while (a >= 0) {
		G.insert(a);
		a = p[a];
	}
	while (b >= 0) {
		if (G.count(b)) {
			return b;
		}
		b = p[b];
	}
	return -1;
}

pair<int, int> dfs(int v,
	vector<vector<int> > const & G,
	vector<int> &tin, 
	vector<int> &tout,
	int &time,
	vector<int> const& p,
	vector<int> const& h,
	bool &is_ans,
	int pred=-1) {

	if (tin[v] >= 0) {
		return { tin[v], v };
	}
	//cerr << v << '\n';
	tin[v] = time++;
	pair<int, int> ans = { inf, -1 };
	vector<pair<int, int> > T;
	for (auto u : G[v]) {
		if (u != pred) {
			bool fl = tin[u] < 0;
			auto z = dfs(u, G, tin, tout, time, p, h, is_ans, v);
			ans = min(ans, z);
			T.push_back(z);
		}
	}
	sort(T.begin(), T.end());
	if (T.size() >= 2 && !is_ans) {
		if (T[0].first < tin[v] && T[1].first < tin[v]) {
			int a = T[0].second;
			int b = T[1].second;
			int s = v;

			int ab = lca(a, b, p);
			int as = lca(a, s, p);
			int sb = lca(s, b, p);

			if (h[ab] >= h[as] && h[ab] >= h[sb]) {
				print_answer(v, ab, G);
			}
			else if (h[sb] >= h[as] && h[sb] >= h[ab]) {
				print_answer(v, sb, G);
			}
			else {
				print_answer(v, as, G);
			}
			is_ans = true;
		}
	}
	tout[v] = time++;
	return ans;
}

int main() {
	freopen("grand.in", "r", stdin);
	freopen("grand.out", "w", stdout);

	int test;
	cin >> test;
	while (test--) {
		int n, m;
		cin >> n >> m;
		vector<vector<int> > G(n);
		for (int i = 0; i < m; i++) {
			int a, b;
			cin >> a >> b;
			a--, b--;
			G[a].push_back(b);
			G[b].push_back(a);
		}
		vector<int> tin(n, -1);
		vector<int> tout(n, -1);
		vector<int> h(n, -1);
		vector<int> p(n);
		bool is_ans = false;
		int time = 0;

		for (int i = 0; i < n && !is_ans; i++) {
			dfs_1(i, -1, h, p, 0, G);
			dfs(i, G, tin, tout, time, p, h, is_ans);
		}
		if (!is_ans) {
			cout << "-1\n";
		}
	}

	return 0;
}
