#include <iostream>
#include <cmath>
#include <cstdio>
#include <vector>
#include <set>
#include <map>
#include <algorithm>
#include <memory.h>
#include <string>
#include <cstring>
#include <cassert>

using namespace std;

#define forn(i, n) for (int i = 0; i < int(n); ++i)
#define ford(i, n) for (int i = int(n) - 1; i >= 0; --i)
#define mp(x, y) make_pair(x, y)
#define pb push_back
#define sz(c) int((c).size())
#define all(c) (c).begin(), (c).end()
#define fst first
#define snd second

#ifdef LOCAL
#define eprintf(args...) fprintf(stderr, args)
#else
#define eprintf(args...) ;
#endif

#define FILE_NAME "grand"

using ll = long long;
using vi = vector<int>;
using vvi = vector<vi>;
using pii = pair<int, int>;

const int MAXN = 100500;

int n, m;
vi g[MAXN];
int used[MAXN];
int tmr, tin[MAXN], tout[MAXN], par[MAXN];
set<pair<int, pii>> ups[MAXN];

bool read() {
    if  (scanf("%d%d", &n, &m) < 2) {
        return 0;
    }

    forn(i, n) {
        g[i].clear();
    }
    forn(i, m) {
        int a, b;
        scanf("%d%d", &a, &b);
        a--, b--;
        g[a].pb(b);
        g[b].pb(a);
    }

    return 1;
}

void update_with(int to, int from) {
    for (auto elem: ups[from]) {
        ups[to].insert(elem);
    }
    while (!ups[to].empty() && sz(ups[to]) > 2) {
        ups[to].erase(*ups[to].begin());
    }
}

int id_tmr, id_used[MAXN];

void count_times(int v, int pv) {
    id_used[v] = id_tmr;
    tin[v] = tmr++;
    for (auto w: g[v]) {
        if (w == pv) {
            continue;
        }

        if (id_used[w] != id_tmr) { // new
            par[w] = v;
            count_times(w, v);
        }
    }
    tout[v] = tmr++;
}

int is_parent_of(int v, int u) {
    return tin[v] <= tin[u] && tout[u] <= tout[v];
}

int find_lca(int v, int u) {
    if (is_parent_of(v, u)) {
        return v;
    }
    if (is_parent_of(u, v)) {
        return u;
    }

    while (true) {
        v = par[v];
        assert(v != -1);
        if (is_parent_of(v, u)) {
            break;
        }
    }

    return v;
}

int S, T;
vi paths[3];

void populate_path(int from, int to, vi &where) {
    assert(is_parent_of(to, from));
    where.pb(from);
    while (from != to) {
        from = par[from];
        where.pb(from);
    }
}

void add_path(int id, int from, int to) {
    vi res;
    if (!is_parent_of(to, from)) {
        assert(is_parent_of(from, to));
        populate_path(to, from, res);
        reverse(all(res));
    } else {
        populate_path(from, to, res);
    }

    for (auto v: res) {
        paths[id].pb(v);
    }
}

bool dfs(int v, int pv) {
    id_used[v] = id_tmr;
    ups[v].clear();

    for (auto w: g[v]) {
        if (w == pv) {
            continue;
        }

        if (id_used[w] != id_tmr) { // new
            if (dfs(w, v)) {
                return 1;
            }

            // check edge v -> w
            assert(0 <= sz(ups[w]) && sz(ups[w]) <= 2);

            if (sz(ups[w]) == 2) {
                int d[2], u[2], ptr = 0;
                for (auto e: ups[w]) {
                    d[ptr] = e.snd.fst;
                    u[ptr] = e.snd.snd;
                    assert(tin[u[ptr]] == -e.fst);
                    ptr++;
                }

                if (tin[u[0]] <= tin[v] && tin[u[1]] <= tin[v]) {

                    int LCA = find_lca(d[0], d[1]);

                    if (LCA == d[1]) {
                        swap(u[0], u[1]);
                        swap(d[0], d[1]);
                    }

                    assert(is_parent_of(u[0], v));
                    assert(is_parent_of(u[1], v));
                    assert(d[0] != u[0]);
                    assert(d[1] != u[1]);

                    S = LCA;
                    T = (tin[u[0]] <= tin[u[1]] ? u[1] : u[0]);

                    add_path(0, S, T);

                    if (LCA != d[0]) { // non-trivial LCA
                        add_path(1, S, d[0]);
                        add_path(1, u[0], T);

                        add_path(2, S, d[1]);
                        add_path(2, u[1], T);
                    } else {
                        if (T == u[0]) {
                            paths[1].pb(S);
                            paths[1].pb(T);

                            add_path(2, S, d[1]);
                            add_path(2, u[1], T);
                        } else {
                            add_path(1, d[0], d[1]);
                            paths[1].pb(T);

                            paths[2].pb(S);
                            add_path(2, u[0], T);
                        }
                    }
/*               
                    cout << "WE WIN ON EDGE " << v << " -> " << w << endl;
                    forn(i, 2) {
                        cout << d[i] << ' ' << u[i] << '\n';
                    }
*/
                    return 1;
                }
            }

            // update uppers
            update_with(v, w);
        } else {
            if (tin[w] < tin[v]) { // v up to w; it is inverted
                int d = v;
                int u = w;
                auto elem = mp(-tin[u], mp(d, u));
                ups[v].insert(elem);
                while (sz(ups[v]) > 2) {
                    ups[v].erase(*ups[v].begin());
                }    
            }
        }
    }

    return 0;
}

bool solve_for_component(int root) {
    id_tmr++;
    par[root] = -1;
    count_times(root, -1);
/*
    cout << "GRAPH:\n";
    forn(i, n) {
        cout << par[i] << " -> " << i << endl;
    }
*/
    id_tmr++;    
    if (dfs(root, -1)) {
        printf("%d %d\n", S + 1, T + 1);
        forn(i, 3) {
            printf("%d", sz(paths[i]));
            for (auto v: paths[i]) {
                printf(" %d", v + 1);
            }
            cout << '\n';
        }

        return 1;
    }
    return 0;
}

void solve() {
    id_tmr = 0;
    tmr = 0;
    forn(i, n) {
        used[i] = 0;
        id_used[i] = 0;
    }
    forn(i, n) {
        tin[i] = -1;
    }

    S = T = -1;
    forn(i, 3) {
        paths[i].clear();
    }

    forn(i, n) {
        if (!used[i]) {
            if (solve_for_component(i)) {
                return;
            }
        }
    }

    cout << -1 << '\n';
}


int main() {
#ifdef LOCAL
    freopen(FILE_NAME ".in", "r", stdin);
//    freopen(FILE_NAME ".out", "w", stdout);
#else
    freopen(FILE_NAME ".in", "r", stdin);
    freopen(FILE_NAME ".out", "w", stdout);
#endif

    int T;
    cin >> T;
    forn(tt, T) {
        assert(read());
        solve();
    }

    return 0;
}





