#include <bits/stdc++.h>

using std::cin;
using std::cout;
using std::cerr;

using std::pair;
using std::make_pair;

using std::vector;
using std::string;
using std::stack;

using std::fixed;
using std::ios_base;

#define pb push_back
#define mp make_pair
#define all(x) x.begin(), x.end()

const int MAXN = 100005;
const int INF = 1e9;

struct Edge {
    int v, u;
};

int T;

int n = 0, m = 0;
vector<int> g[MAXN];
vector<int> g2[MAXN];

int col[MAXN];
int depth[MAXN];
int fup[MAXN];

bool is_point[MAXN];

int comp_sz = 0;
vector<Edge> comp[MAXN];

void get_points(int v, int p) {
    col[v] = 1;
    fup[v] = depth[v];

    int visited = 0;
    bool exists = false;
    for (int u : g[v]) {
        if (col[u] == 0) {
            depth[u] = depth[v] + 1;
            get_points(u, v);
            fup[v] = std::min(fup[v], fup[u]);
            visited++;
            exists |= fup[u] >= depth[v];
        } else {
            if (u != p)
                fup[v] = std::min(fup[v], depth[u]);
        }
    }

    if (exists && (p != -1 || visited > 1)) {
        is_point[v] = true;
    }
}

void add_component(int v, int u, stack<Edge>& st) {
    while (!st.empty()) {
        int v2 = st.top().v;
        int u2 = st.top().u;
        comp[comp_sz].pb(st.top());
        st.pop();
        if (v == v2 && u == u2)
            break;
    }
    comp_sz++;
}

void get_components(int v, int p, stack<Edge>& st) {
    col[v] = 2;
    for (int u : g[v]) {
        if (u != p)
            st.push({v, u});
        if (col[u] == 1) {
            get_components(u, v, st);
            if (fup[u] >= depth[v]) {
                add_component(v, u, st);
            }
        }
    }
    col[v] = 3;
}

int get_cycle(int v, int p, vector<int>& cycle) {
    col[v] = 4;
    for (int u : g2[v]) {
        if (u == p)
            continue;
        int type = 0;
        if (col[u] != 4) {
            if ((type = get_cycle(u, v, cycle))) {
                if (type == 1)
                    cycle.pb(u);
                if (v == cycle.front())
                    type = 2;
                return type;
            }
        } else {
            cycle.pb(u);
            return 1;
        }
    }
    return 0;
}

void clear_data() {
    for (int i = 0; i < n; i++) {
        g[i].clear();
        col[i] = 0;
        depth[i] = 0;
        is_point[i] = false;
        g2[i].clear();
    }
    for (int i = 0; i < comp_sz; i++) {
        comp[i].clear();
    }
    comp_sz = 0;
}

bool find_path(int v, int& st, int& prev, int& next, vector<int>& path) {
    col[v] = 7;
    for (int u : g[v]) {
        if (col[u] < 6) {
            if (find_path(u, st, prev, next, path)) {
                path.pb(u);
                return true;
            }
        } else {
            if (col[u] == 6 && (v != st || (u != next && u != prev))) {
                path.pb(u);
                return true;
            }
        }
    }
    return false;
}

void read_data() {
    cin >> n >> m;
            
    for (int i = 0; i < m; i++) {
        int v, u;
        cin >> v >> u;
        v--, u--;
        g[v].pb(u);
        g[u].pb(v);
    }
}

void solve() {
    clear_data();
    read_data();

    for (int i = 0; i < n; i++)
        if (col[i] == 0)
            get_points(i, -1);

    stack<Edge> st;
    for (int i = 0; i < n; i++) {
        if (col[i] == 1)
            get_components(i, -1, st);
    }                
    
    /*
    for (int i = 0; i < comp_sz; i++) {
        cout << "comp " << i + 1 << ":\n";
        for (Edge e : comp[i]) {
            cout << e.v << ' ' << e.u << '\n';                    
        }
        cout << '\n';
    }
    */

    vector<int> cycle, path;
    bool found = false;
    for (int i = 0; i < comp_sz; i++) {
        for (Edge& e : comp[i]) {
            g2[e.v].pb(e.u);
            g2[e.u].pb(e.v);
        }

        if (!comp[i].empty()) {
            get_cycle(comp[i][0].v, -1, cycle);
            for (int v : cycle)
                col[v] = 6;   
            for (int i = 0; i < cycle.size(); i++) {
                int v = cycle[i];
                int prev = i == 0 ? cycle.back() : cycle[i - 1];
                int next = i + 1 == cycle.size() ? cycle.front() : cycle[i + 1];
                if (g2[v].size() > 2) {
                    find_path(v, v, prev, next, path);
                    path.pb(v);
                    found = true;
                    break;  
                }
            }
            if (found)
                break;            
        }

        for (Edge& e : comp[i]) {
            g2[e.v].clear();
            g2[e.u].clear();
            col[e.v] = col[e.u] = 3;
        }
        cycle.clear();
    }

    if (!found) {
        cout << -1 << '\n';
        return;
    }

    vector<int> path2, path3;
    int sz = cycle.size();
    for (int i = 0; i < sz; i++) {
        int v = cycle[i]; 
        if (v == path.front()) {
            for (int j = 0; j < sz; j++) {
                path2.pb(cycle[(i + j) % sz]);
                if (cycle[(i + j) % sz] == path.back())
                    break;
            }
            for (int j = 0; j < sz; j++) {
                path3.pb(cycle[(i - j + sz) % sz]);
                if (cycle[(i - j + sz) % sz] == path.back())
                    break;
            }
        }
    }


    cout << path.front() + 1 << ' ' << path.back() + 1 << '\n';
    cout << path.size() << ' ';
    for (int v : path)
        cout << v + 1<< ' ';
    cout << '\n';
    cout << path2.size() << ' ';
    for (int v : path2) 
        cout << v + 1 << ' ';
    cout << '\n';
    cout << path3.size() << ' ';
    for (int v : path3) 
        cout << v + 1 << ' ';
    cout << '\n';
}

int main() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    cout.precision(10);
    cout << fixed;

#ifndef LOCAL
    #define FILE "grand"
    freopen(FILE".in", "r", stdin);
    freopen(FILE".out", "w", stdout);
#endif

    cin >> T;
    while (T--) {
        solve();
    }

    return 0;
}